# BruteBuster by Cyber Security Consulting (www.csc.bg)
"""
Module preventing Brute Force attacks against
django.contrib.auth.authenticate()
"""

version = '0.1.8'

from django.contrib import auth
from BruteBuster.decorators import protect_and_serve

# here we override the default authenticate method with the decorated version
auth.authenticate  = protect_and_serve (auth.authenticate)



# BruteBuster by Cyber Security Consulting (www.csc.bg)

"""
BruteBuster DB model.

It makes use of the BB_MAX_FAILURES and BB_BLOCK_INTERVAL values in
settings.py. If these are not present, default values are used.

Blocks are applied against the unique combination of (user,IP). This means that
a bruteforce attacker won't stop the legitimate user from logging in (assuming
they are using different IP addresses).

The easiest way to remove a block is to delete the FailedAttempt record (e.g.
via the admin).
"""

from django.db import models
from datetime import datetime, timedelta
from django.conf import settings

# default values that can be overriden in settings.py
BB_MAX_FAILURES = int(getattr(settings,'BB_MAX_FAILURES', 5))
BB_BLOCK_INTERVAL = int(getattr(settings,'BB_BLOCK_INTERVAL', 3))

class FailedAttempt (models.Model):
    username = models.CharField('Username',max_length=255)
    IP =  models.IPAddressField('IP Address',null=True)
    failures = models.PositiveIntegerField('Failures',default=0)
    timestamp = models.DateTimeField('Last failed attempt',auto_now=True)

    def too_many_failures (self):
        """Check if the minumum number of failures needed for a block
        is reached"""
        return self.failures >= BB_MAX_FAILURES

    def recent_failure (self):
        """Checks if the timestamp one the FailedAttempt object is
                recent enough to result in an increase in failures"""
        return datetime.now() < self.timestamp + timedelta(minutes=BB_BLOCK_INTERVAL)

    def blocked (self):
        """Shortcut function for checking both too_many_failures and recent_failure """
        return self.too_many_failures() and self.recent_failure()
    blocked.boolean = True

    def __unicode__(self):
        return u'%s (%d failures until %s): ' % (self.username,self.failures, self.timestamp)

    class Meta:
        ordering = ['-timestamp']
        unique_together = (("username", "IP"),)



# BruteBuster by Cyber Security Consulting (www.csc.bg)

"""Decorators used by BruteBuster"""

from BruteBuster.models import FailedAttempt
from BruteBuster.middleware import get_request

def protect_and_serve (auth_func):
    """
    This is the main code of the application. It is meant to replace the
    authentication() function, with one that records failed login attempts and
    blocks logins, if a threshold is reached
    """

    if hasattr(auth_func,'__BB_PROTECTED__'):
        # avoiding multiple decorations
        return auth_func
    
    def decor (*args, **kwargs):
        """
        This is the wrapper that gets installed around the default
        authentication function.
        """
        user = kwargs.get('username','')
        if not user:
            raise ValueError ('BruteBuster cannot work with authenticate functions that do not include "username" as an argument')
        
        request = get_request()
        if request:
            # try to get the remote address from thread locals
            IP_ADDR = request.META.get('REMOTE_ADDR',None)
        else:
            IP_ADDR = None
        
        try:
            fa = FailedAttempt.objects.filter(username=user,IP=IP_ADDR)[0]
            if fa.recent_failure():
                if fa.too_many_failures() :
                    # we block the authentication attempt because
                    # of too many recent failures
                    fa.failures += 1
                    fa.save()
                    return None
            else:
                # the block interval is over, so let's start
                # with a clean sheet
                fa.failures = 0
                fa.save()
        except IndexError:
            # No previous failed attempts
            fa = None

        result = auth_func (*args, **kwargs)
        if result:
            # the authentication was successful - we do nothing
            # special
            return result
        # the authentication was kaput, we should record this
        fa = fa or FailedAttempt (username=user,IP=IP_ADDR,failures=0)
        fa.failures+=1
        fa.save()
        # return with unsuccessful auth
        return None

    decor.__BB_PROTECTED__ = True
    return decor



# BruteBuster by Cyber Security Consulting (www.csc.bg)

"""Admin settings for the BruteBuster module"""

from django.contrib import admin
from BruteBuster.models import FailedAttempt

class AdminFailedAttempt (admin.ModelAdmin):
    list_display = ('username', 'IP' ,'failures', 'timestamp','blocked')
    search_fields = ('username','IP',)


admin.site.register(FailedAttempt, AdminFailedAttempt)



#!/usr/bin/env python
#import ez_setup
#ez_setup.use_setuptools()
from setuptools import setup, find_packages

setup(\
    name = "django-brutebuster",
    version = "0.1.8",
    packages = find_packages(),
    author = "Cyber Security Consulting Ltd.",
    author_email = "contact@csc.bg",
    description = "Pluggable Django application providing password bruteforce protection",
    url = "http://code.google.com/p/django-brutebuster/",
    include_package_data = True,
    zip_safe = False
)



# BruteBuster by Cyber Security Consulting (www.csc.bg)

"""
Brutebuster needs access to the REMOTE_IP of the incoming request. We're doing
this by adding the request object to the thread_local space
"""
import sys

try:
    from threading import local
except ImportError:
    from django.utils._threading_local import local

_thread_locals = local()

def get_request():
    return getattr(_thread_locals, 'request', None)

class RequestMiddleware (object):
    """Provides access to the request object via thread locals"""
    def process_request (self, request):
        _thread_locals.request = request
